/* 
 * EDOMI beta (Version 1.57) - Intelligente Steuerung und Visualisierung KNX-basierter Elektro-Installationen.
 * @link http://www.edomi.de 
 * @author Dr. Christian Gärtner <dr.christian.gaertner@googlemail.com> 
 * @copyright 2015,2016,2017 Dr. Christian Gärtner 
 * @license Freeware 
 *
 *     This file and all other files are part of EDOMI.
 *
 *     EDOMI is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *     Diese und alle anderen Dateien sind Teil von EDOMI.
 *
 *     EDOMI wird in der Hoffnung, dass es nützlich sein wird, aber
 *     OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *     Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
*/ 
/*
============================================================================
Initialisierung
============================================================================
*/

function firstinit(visuid,newSid,version) {
	//dies wird NUR EINMALIG beim allerersten Start aufgerufen (also nach dem Laden der index.php)
	
	//--- ggf. anpassen ---
	displayPixelRatio=2;			//Faktor für die virtuelle Auflösung eines CANVAS (z.B. für Retina-Displays sollte der Faktor mindestens 2 betragen)
									//benötigt für: Diagramme, Rundinstrumente

	visu_cssbuffer=true;			//true=CSS-Style von Visuelementen nur bei Änderung setzen (vermeidet u.U. flackern bei einigen Browsern)
	//---------------------

	sid=newSid;
	edomiVersion=version;

	visu_visuid=null;				//aktuelle VisuID (wird erst in app0.php/initVisu gesetzt)
	visu_startPageId=null;			//Startseite
	visu_sspageid=null;				//Bildschirmschoner-Seiten-ID
	visu_sstimeout=null; 			//nach dieser Zeit (Minuten!) wird der Bilderschirmschoner aufgerufen

	visu_indiColor='#80e000';		//Farbe für alle Indikatoren (Klicks, ZSU, etc.)
	visu_indiColorText='inherit';	//Text-Farbe für einige Indikatoren (Tastatureingabe, Drehregler, etc.)

	visu_touchMode=0;				//Touch oder Click? (0=Automatisch, 1=Klick, 2=Touch)
	visu_allowScrolling=false;		//iOS: Scrollen erlauben?

	visu_isMobile=(navigator.userAgent.match(/iPhone|iPad|iPod|iOS|Android/i)?true:false);	//true=Mobile-Device

	visu_sound=false;				//Audio-Objekt
	visu_soundEnabled=0;			//0=Sound wird nicht benötigt, 1=wird benötigt ist aber noch nicht initialisiert (iOS), 2=steht zu Verfügung und ist initialisiert
	visu_textToSpeech=false; 		//SpeechSynthesisUtterance-Objekt
	visu_textToSpeechEnabled=0;		//0=TextToSpeech wird nicht benötigt, 1=wird benötigt ist aber noch nicht initialisiert (iOS), 2=steht zu Verfügung und ist initialisiert

	preload_images_max=0;			//Preload-Bilder (Anzahl, nur temporär für den Preloadvorgang!)
	preload_imagesFile=new Array();	//Preload-Bilder (Dateinamen, nur temporär für den Preloadvorgang!)

	visu_socket=null;				//Websocket
	visu_interrupt={id:null,lastTs:null};	//Timer

	initVisu(visuid);
}

function initVisu(visuid) {
	//wird beim Start und Fehler (=Neustart) aufgerufen

	windowZindex=0; 				//Fenster-Verwaltung

	errorTimer=null; 				//Timer für Fehlerbehandlung (serverNotReady)
	screensaverTimer=null; 			//Timer für Bildschirmschoner
	longClickTimer=null;			//Timer für Longclick
	screensaverActive=false;		//Flag: Bildschirmschoner aktiv/inaktiv
	
	//alle Timer löschen und Timer-Funktion starten
	window.clearInterval(visu_interrupt.id);
	visu_interrupt={id:window.setInterval("check_timeout()",5),lastTs:Date.now()};
	visu_timeouts=new Array();

	//für Controls mit Drag-Funktion (Mausposition und KO-Intervall)
	element_drag={id:0,koeditvalue:null,kolastvalue:null,koblock:0,kotimer:null,liveblock:0,livetimer:null,startx:null,starty:null,valuex:null,valuey:null,lastcx:null,lastcy:null};
	
	visu_showWarningErr=0;			//Fehlerhinweis quittieren: Enthält die Anzahl der aktuell quittierten Fehler, -1=deaktiviert

	ajax("initVisu",0,visuid,0,"","");
}

function prepareVisu() {
	document.getElementById("waitanim").style.borderLeftColor="#20e000";
	document.getElementById("waitanim").style.borderRightColor="#20e000";
	if (preload_imagesFile.length>0) {
		document.getElementById("preload").style.display="block";
		preload_images_max=preload_imagesFile.length;
		preloadImagesLoad();
	} else {
		startVisu();
	}
}

function preloadImagesLoad() {
	if (preload_imagesFile.length>0) {
		var id=preload_imagesFile[0].match(/visu\/img\/img-([0-9]*)./);
		if (id && id[1]>0) {
			var obj=new Image();			
			obj.onload=function(){preloadImagesReady();}
			obj.onerror=function(){preloadImagesReady();}
			obj.src=preload_imagesFile.shift();
		} else {
			preload_imagesFile.shift();
			preloadImagesReady();		
		}
	} else {
		startVisu();
	}
}

function preloadImagesReady() {
	var t=parseInt((preload_images_max-preload_imagesFile.length)*100/preload_images_max);
	document.getElementById("preload").style.background="-webkit-linear-gradient(left,#ffffff 0%,#ffffff "+t+"%,#606060 "+t+"%,#606060 100%)";
	preloadImagesLoad();
}

function startVisu() {
	document.getElementById("waitanim").style.borderLeftColor="#e00000";
	document.getElementById("waitanim").style.borderRightColor="#e00000";
	document.getElementById("wait").style.display="none";
	document.getElementById("preload").style.display="none";
	document.getElementById("windowContainer").style.display="block";
	initScreensaverTimer();
	openPage(visu_startPageId);
}

function jsLogin(visuid,visus) {
	//Logindialog (modal)	
	gotoStart();
	var loginWinId="login";
	var n="<div id='loginform' class='appWindowLogin' onkeydown='if (event.keyCode==13) {ajax(\"login\",0,\"\",\"\",\"\",controlGetFormData(\"loginform\"));}' style='width:200px;'>";
		n+="<div style='margin-bottom:10px;'><b>Login</b></div>";
		n+="<select id='"+loginWinId+"-loginform-fd0' class='loginSelect' data-type='1' style='margin-bottom:15px;'>"+visus+"</select>";
		n+="<input type='text' id='"+loginWinId+"-loginform-fd1' data-type='1' value='' maxlength='30' class='loginInput' autofocus style='width:100%; height:25px; padding:5px; border-color:transparent;'></input><br>";
		n+="<input type='password' id='"+loginWinId+"-loginform-fd2' data-type='1' value='' maxlength='30' class='loginInput' style='width:100%; height:25px; padding:5px; border-color:transparent;'></input><br>";
		n+="<br>";
		n+="<button id='"+loginWinId+"-loginform-fd3' class='loginCheckbox' onClick='checkboxToggle(this);' data-type='5' data-list='normal|Neuladen' data-value='"+((visu_isMobile)?"1":"0")+"' style='float:left;'></button>";
		n+="<div class='loginButton' onClick='ajax(\"login\",0,\"\",\"\",\"\",controlGetFormData(\"loginform\"));' style='float:right; border-color:transparent;'>Ok</div>";
	n+="</div>";
	
	//POST-Login (für alterative Login-Methode unter iOS & Co.)
	n+="<form id='loginform-post' action='index.php' method='post'><input type='hidden' id='loginvisuid' name='loginvisuid' value=''></input><input type='hidden' id='loginsid' name='loginsid' value=''></input></form>";

	document.getElementById(loginWinId).innerHTML=n;
	checkboxInit(document.getElementById(loginWinId+"-loginform-fd3"));
	document.getElementById(loginWinId).style.display="block";
}


/*
============================================================================
Sound & Speech
============================================================================
*/

function visuSoundInit(obj) {
	if (visu_soundEnabled==1) {
		if (!visu_sound) {visu_sound=new Audio("../shared/etc/snd-empty.mp3");}	//Dummy
		visu_sound.type="audio/mpeg";
		visu_sound.preload="none";
		visu_sound.load();
		visu_sound.autoplay=false;
		visu_sound.loop=false;
		visu_sound.onplay=function(event){
			visu_soundEnabled=2;
			if (obj && visu_soundEnabled!=1 && visu_textToSpeechEnabled!=1) {obj.style.display="none";}
		}
		visu_sound.play();
	}
}

function visuSoundPlay(sndId,sndUrl,sndLoop) {
	if (visu_sound && visu_soundEnabled==2) {
		if (sndId>0) {sndUrl="../data/liveproject/visu/etc/snd-"+sndId+".mp3";}
		visu_sound.src=sndUrl; 
		visu_sound.load();
		visu_sound.loop=sndLoop;
		visu_sound.play();
	}
}

function visuSoundStop() {
	if (visu_sound && visu_soundEnabled==2) {
		visu_sound.src="../shared/etc/snd-empty.mp3";
		visu_sound.load();
		visu_sound.pause();
	}
}

function visuTextToSpeechInit(obj) {
	if (visu_textToSpeechEnabled==1 && ("speechSynthesis" in window)) {
		if (!visu_textToSpeech) {visu_textToSpeech=new SpeechSynthesisUtterance();}
		visu_textToSpeech.onstart=function(event){
			visu_textToSpeechEnabled=2;
			if (obj && visu_soundEnabled!=1 && visu_textToSpeechEnabled!=1) {obj.style.display="none";}
		}
		visu_textToSpeech.text=" ";	//Dummy (darf nicht leer sein, muss bei Init ausgesprochen werden!)
		window.speechSynthesis.speak(visu_textToSpeech);
	}
}	

function visuTextToSpeechPlay(voice,language,rate,pitch,msg) {
	if (visu_textToSpeech && visu_textToSpeechEnabled==2 && msg!="") {
		visu_textToSpeech.text=msg;
		if (voice!="") 		{visu_textToSpeech.voice=voice;}	//### Voice wird noch nicht wirklich unterstützt...
		if (language!="") 	{visu_textToSpeech.lang=language;}
		if (rate!="") 		{visu_textToSpeech.rate=rate;}
		if (pitch!="") 		{visu_textToSpeech.pitch=pitch;}
		window.speechSynthesis.speak(visu_textToSpeech);
	}
}

function visuTextToSpeechStop() {
	if (visu_textToSpeech && visu_textToSpeechEnabled==2) {
		window.speechSynthesis.cancel();
	}
}


/*
============================================================================
Visuseiten & Popups
============================================================================
*/

function openPage(pageid,sspage,userClick) {
	if (userClick!==true) {userClick=false;}
	//Visuseite/Popup aufrufen
	if (visu_socket) {visu_socket.request_initPage(pageid,userClick);}
}

function reloadPage() {
	//### ungenutzt
	//aktuelle Visuseite (nicht Popup!) neu laden (KOs updaten)
	var obj=document.getElementById("w0");
	if (obj && obj.dataset.pageid>0) {
		openPage(obj.dataset.pageid);
		if (visu_socket) {visu_socket.request_initPage(pageid);}
	}
}

function focusPopup(pageid) {
	//existiert bereits ein Popup(!) mit dieser pageid, wird das Popup in den Vordergrund geholt
	//return: true=Popup ist bereits geöffnet, false=Popup existiert nicht
	var win=document.getElementById("windowContainer").childNodes;
	for (var t=0;t<win.length;t++) {
		if (win[t].dataset.pagetyp==1 && win[t].dataset.pageid==pageid) {
			focusWindow(win[t].id);
			return true;
		}
	}
	return false;
}

function closePopup(winId,userClick) {
	if (userClick!==true) {userClick=false;}
	if (winId!="w0") {
		cancelAll();
		//ggf. AutoClose-Timer löschen
		var obj=document.getElementById(winId);
		if (obj) {
			obj.dataset.autoclosetimer=clear_timeout(obj.dataset.autoclosetimer);
			if (visu_socket) {visu_socket.request_closePopup(obj.dataset.pageid,userClick);}
		}
		closeWindow(winId);
	}
	cancelTimeouts();
}

function closeAllPopups() {
	var tmp=new Array();
	var win=document.getElementById("windowContainer").childNodes;
	for (var t=0;t<win.length;t++) {
		if (win[t].dataset.pagetyp==1) {tmp.push(win[t].id);}
	}
	
	//alle Popups schliessen (nur DOM)
	for (var t=0;t<tmp.length;t++) {
		if (tmp[t]!="w0") {
			//ggf. AutoClose-Timer löschen
			var obj=document.getElementById(tmp[t]);
			if (obj) {obj.dataset.autoclosetimer=clear_timeout(obj.dataset.autoclosetimer);}
			closeWindow(tmp[t]);
		}
	}
}


/*
============================================================================
Bildschirmschoner
============================================================================
*/

function startScreensaver() {
	if (visu_sstimeout>0 && visu_sspageid>0) {
		screensaverActive=true;
		openPage(visu_sspageid,true);
	}
}

function initScreensaverTimer() {
	//wird bei jeder User-Aktion (Click) von checkClick() aufgerufen, und bei jedem Seitenwechsel (normal und Popup, damit ein Visualarm den Bildschirmschoner deaktiviert)
	screensaverActive=false;
	screensaverTimer=clear_timeout(screensaverTimer);
	if (visu_sstimeout>0 && visu_sspageid>0) {
		screensaverTimer=set_timeout("startScreensaver();",visu_sstimeout*60*1000,false,"screensaverTimer");
	}
}


/*
============================================================================
Warnungen & Fehlerbehandlung
============================================================================
*/

function showWarnings(arr) {
	var n="";
	if (visu_soundEnabled==1 || visu_textToSpeechEnabled==1) {n+="<div onClick='ackWarningSound(this);' class='visuWarning' style='color:#000000; background:#80e000; pointer-events:auto;'>&#9651;<br>Tonausgabe aktivieren</div>";}	//Achtung: onClick ist wichtig für iOS (nicht downEvent)!
	if (visu_showWarningErr>=0 && parseInt(arr[3])>visu_showWarningErr) {n+="<div onClick='ackWarningErr(this,"+parseInt(arr[3])+");' class='visuWarning' style='color:#ffffff; background:#ff4000; pointer-events:auto;'>&#9651;<br>"+arr[3]+" FEHLER</div>";}	//dto.
	if (arr[0]==1) {n+="<div class='visuWarning'>&#9651;<br>CPU</div>";}
	if (arr[1]==1) {n+="<div class='visuWarning'>&#9651;<br>RAM</div>";}
	if (arr[2]==1) {n+="<div class='visuWarning'>&#9651;<br>HDD</div>";}
	if (n!="") {
		document.getElementById("warn").innerHTML=n;
		document.getElementById("warn").style.display="inline";
	} else {
		document.getElementById("warn").style.display="none";
	}	
}

function ackWarningErr(obj,n) {
	obj.style.display="none";
	visu_showWarningErr=n;
}

function ackWarningSound(obj) {
	visuSoundInit(obj);
	visuTextToSpeechInit(obj);
}

function cancelAll() {
	//Drag-Operationen ggf. absenden
	controlResetDrag();

	//Longclick abbrechen
	cancelLongClick();

	//Globale Timer löschen
	errorTimer=clear_timeout(errorTimer);
	element_drag.kotimer=clear_timeout(element_drag.kotimer);
	element_drag.livetimer=clear_timeout(element_drag.livetimer);
}

function cancelTimeouts() {
	//alle Timer löschen, die einem DOM-Element zugeordnet sind, das nach(!) dem Seitenaufbau/etc. nicht mehr vorhanden ist
	for (var t=0;t<visu_timeouts.length;t++) {
		if (visu_timeouts[t].ts!==false && visu_timeouts[t].objid!==false) {
			var obj=document.getElementById(visu_timeouts[t].objid);
			if (!obj) {clear_timeout(t);}
		}
	}
}

function gotoStart() {
	//alles auf Neustart
	cancelAll();
	if (visu_socket) {
		visu_socket.request_close();
		visu_socket=null;
	}
	screensaverTimer=clear_timeout(screensaverTimer);
		
	//alle Fenster entfernen
	clearObject("windowContainer",0);

	//alle Warnungen ausblenden
	document.getElementById("warn").style.display="none";

	//alle ungültigen Timer löschen
	cancelTimeouts();

	//Sounds stoppen
	visuSoundStop();
	visuTextToSpeechStop();

	//wait-Animation einblenden
	document.getElementById("wait").style.display="block";
}

function serverNotReady(visuid) {
	//Server ist nicht bereit (db.sysInfo)
	//d.h. php-Serverscripte starten gerade neu (z.B. beim Compilieren...)
	//=> Visu immer wieder starten, bis es klappt

	gotoStart();

	//kompletter Neustart nach 3 Sekunden
	errorTimer=set_timeout("initVisu("+visuid+");",3*1000,false,"errorTimer");
}


/*
============================================================================
Fenster-Verwaltung
============================================================================
*/

function openWindow(xsize,ysize,xpos,ypos) {
	var winID=newWindow();

	if (winID=="w0") { //w0 (Visu-Seite)
		var n="<div id='"+winID+"-page' class='appWindowNormal' style='width:"+xsize+"px; height:"+ysize+"px; box-shadow:none; -webkit-animation:none;'></div>";
		document.getElementById(winID).style.background="none";
	} else { //w1..x (Popups)
		if (xpos!==undefined && ypos!==undefined) {
			var n="<div id='"+winID+"-page' class='appWindowNormal' style='-webkit-animation:none; position:absolute; left:"+xpos+"px; top:"+ypos+"px; width:"+xsize+"px; height:"+ysize+"px; border-radius:3px;'></div>";
		} else {
			var n="<table width='100%' height='100%' cellpadding='0' cellspacing='0' border='0'><tr valign='middle'><td>";
			n+="<div id='"+winID+"-page' class='appWindowNormal' style='width:"+xsize+"px; height:"+ysize+"px; border-radius:3px;'></div>";
			n+="</td></tr></table>";
		}
	}
	document.getElementById(winID).innerHTML=n;
	return winID;
}

function focusWindow(winID) {
	//Popup on top bringen
	if (winID!=null) {
		var win=document.getElementById(winID);

		//neuen winID-zindex = ehemals größer zindex
		var maxZ=0;
		var wins=document.getElementById("windowContainer").childNodes;
		for (var t=0;t<wins.length;t++) {
			if (wins[t].style.zIndex>maxZ) {maxZ=parseInt(wins[t].style.zIndex);}
		}

		//alle zindex, die größer sind als win-zindex: dec()
		for (var t=0;t<wins.length;t++) {
			if (wins[t].style.zIndex>win.style.zIndex) {wins[t].style.zIndex=parseInt(wins[t].style.zIndex)-1;}
		}

		win.style.zIndex=maxZ;
	}
}

function closeWindow(winID) {
	if (winID!=null) {

		var win=document.getElementById(winID);

		//alle zindex, die größer sind als win-zindex: dec()
		var wins=document.getElementById("windowContainer").childNodes;
		for (var t=0;t<wins.length;t++) {
			if (wins[t].style.zIndex>win.style.zIndex) {wins[t].style.zIndex=parseInt(wins[t].style.zIndex)-1;}
		}
		windowZindex--;

		if (win) {win.parentNode.removeChild(win);}
		if (document.getElementById("shortclick")) {document.getElementById("shortclick").style.display="none";}
	}
}

function newWindow() {
	if (document.getElementById("windowContainer").lastChild) {
		var newID=parseInt(document.getElementById("windowContainer").lastChild.id.replace("w",""))+1;
	} else {
		var newID=0;
	}
	var winID="w"+newID;
	var div=document.createElement("div");
	document.getElementById("windowContainer").appendChild(div);
	div.className="appWindow";
	div.id=winID;
	windowZindex++;
	div.style.zIndex=windowZindex;
	return winID;
}


/*
============================================================================
Timer und Hilfsfunktionen
============================================================================
*/

function check_timeout() {
	var ts=Date.now();
	var diff=ts-visu_interrupt.lastTs;
	for (var t=0;t<visu_timeouts.length;t++) {
		if (visu_timeouts[t].ts!==false) {
			if ((ts-visu_timeouts[t].ts)>=visu_timeouts[t].timeout) {
				visu_timeouts[t].ts=false;
				if (visu_timeouts[t].resetvar) {eval(visu_timeouts[t].resetvar+"=null;");}
				eval(visu_timeouts[t].func);
			}
		}
	}	
	visu_interrupt.lastTs=ts;
}

function set_timeout(func,timeout,objId,resetVar) {
	var ts=Date.now();
	var id=false;
	for (var t=0;t<visu_timeouts.length;t++) {
		if (visu_timeouts[t].ts===false) {
			id=t;
			break;
		}
	}
	if (id!==false) {
		visu_timeouts[id]={func:func,timeout:timeout,ts:ts,objid:objId,resetvar:resetVar};
	} else {
		id=visu_timeouts.push({func:func,timeout:timeout,ts:ts,objid:objId,resetvar:resetVar})-1;
	}
	return id;
}

function clear_timeout(id) {
	if (id!==undefined && parseInt(id)>=0 && parseInt(id)<visu_timeouts.length) {
		visu_timeouts[id].ts=false;
	}	
	return null;
}

function checkClick() {
	//prüft, ob (z.B. auf iOS-Geräten) mehrere Finger ein touch-Event auslösen
	//(wird bei jeder User-Aktion (Click) aufgerufen und setzt daher auch gleich den Bildschirmschoner-Timer zurück)
	//return: true=Singletouch, false=Multitouch
	initScreensaverTimer();
	if (!event) {var event=window.event;}
	if (event && event.touches) {
		if (event.touches.length>1) {
			return false;
		}
	}
	return true;
}

function getPageX(e) {
	//absolute Mouseposition
	if (e.touches) {
		return e.touches[0].pageX;	//e.changedTouches[0].pageX;
	} else {
		return e.pageX;
	}
}

function getPageY(e) {
	//absolute Mouseposition
	if (e.touches) {
		return e.touches[0].pageY;	//e.changedTouches[0].pageY;
	} else {
		return e.pageY;
	}
}

function getClientX(event) {
	//relative Mouseposition (Viewport)
	if (!event) {var event=window.event;}
	if (!event) {return 0;}
	if (event.touches) {
		return event.touches[0].clientX;	//event.changedTouches[0].clientX;
	} else {
		return event.clientX;
	}
}

function getClientY(event) {
	//relative Mouseposition (Viewport)
	if (!event) {var event=window.event;}
	if (!event) {return 0;}
	if (event.touches) {
		return event.touches[0].clientY;	//event.changedTouches[0].clientY;
	} else {
		return event.clientY;
	}
}


function disableScrolling() {
	if (!event) {var event=window.event;}
	event.preventDefault(); //verhindert das Scrollen auf iOS
}

function getDivPosition(obj) {
	//Liefert die absolute Positions eines Elements zurück (klettert quasi den DOM-Baum nach oben)
	//return: Objekt (z.B. Position.x und Position.y)
	var xPos=0;
	var yPos=0;
	while(obj) {
		xPos+=obj.offsetLeft;
		yPos+=obj.offsetTop;
		obj=obj.offsetParent;
	}
	return [xPos,yPos]; //{x:xPos,y:yPos};
}

function createNewDiv(parentId,id) {
	//erzeugt neues Div und hängt es an parent
	//parentId: Parent-Objekt-ID
	//id: gewünschte ID des neuen DIVs
	var div=document.createElement('div');
	document.getElementById(parentId).appendChild(div);
	div.id=id;
	return div;
}

function clickCancel() {
	//verhindert bei verschachtelten DIVs, dass das Click-Event vom Parent bearbeitet wird, wenn auf das Child geklickt wird
	if (!event) var event=window.event;
	if (event) {
		if (event.cancelBubble) {event.cancelBubble=true;}
		if (event.stopPropagation) {event.stopPropagation();}
	}
}

function clearObject(objId,mode) {
	//Leert ein Objekt (entspricht quasi obj.innerHTML="")
	//mode: 1=setzt obj.style.display auf "none"
	var obj=document.getElementById(objId);
	if (mode==1) {obj.style.display="none";}
	while (obj.firstChild) {obj.removeChild(obj.firstChild);}
}

function shakeObj(objId) {
	var obj=document.getElementById(objId);
	if (obj) {
		obj.style.webkitAnimation="none";
		obj.style.webkitAnimationPlayState="paused";
		set_timeout("document.getElementById('"+objId+"').style.webkitAnimation='animShake 0.3s linear';document.getElementById('"+objId+"').style.webkitAnimationPlayState='running';",100,objId);
	}
}


/*
============================================================================
Ajax & Websocket
============================================================================
*/

function ajax(cmd,appID,visuID,pageID,data,phpdata) {
	var req=new XMLHttpRequest();
	req.onreadystatechange=function() {
		if (this.readyState==4 && this.status==200) {
			try {
				eval(this.responseText);
			} catch(e) {
				console.log("Ajax-Response fehlerhaft! Error-Msg: "+e.message+" / URL: "+decodeURIComponent(url)+" / Response: "+this.responseText);
			}
		}
		if (this.readyState==4 && this.status!=200) {
			console.log("Ajax-Abruf gescheitert! http-status: "+this.status+" / URL: "+decodeURIComponent(url));
			if (appID==0) { //nur Neustart, wenn grundsätzliche Funktionalität gefährdet ist (also NICHT bei Kamera, Diagramm, etc.)
				serverNotReady(visuID);
			}
		}
	}
	//POST
	var url="apps/app"+appID+".php?cmd="+encodeURIComponent(cmd)+"&appid="+encodeURIComponent(appID)+"&visuid="+encodeURIComponent(visuID)+"&pageid="+encodeURIComponent(pageID)+"&sid="+encodeURIComponent(sid)+"&vid="+encodeURIComponent(edomiVersion);
	req.open("POST",url,true);
	req.timeout=30000; //nach 30s: Timeout
	req.setRequestHeader("Content-type","application/x-www-form-urlencoded");
	req.send("data="+encodeURIComponent(data)+"&phpdata="+encodeURIComponent(phpdata));
}

function class_websocket() {
	var that=this;
	
	this.open=function(serverProtocol,serverIp,serverPort) {
		try {
			socket=new WebSocket(serverProtocol+'://'+serverIp+':'+serverPort);

			socket.addEventListener('open',function (event) {
				//Browser-Exit => Client offline setzen
				window.onbeforeunload=function (event) {
					if (socket.readyState==1) {
						socket.close();
					}
				}
			});
			
			socket.addEventListener('message',function (event) {
				var tmp=JSON.parse(event.data);
				if (tmp[0]=='LOGIN') 		{that.request_login();}
				else if (tmp[0]=='START') 	{prepareVisu();}
				else if (tmp[0]=='CMD') 	{eval(tmp[1]);}
				else if (tmp[0]=='PING') 	{that.request_pong();}
			});

			socket.addEventListener('close',function (event) {
				serverNotReady(visu_visuid);
			});
			
			socket.addEventListener('error',function (event) {
				serverNotReady(visu_visuid);
			});
		
		} catch (exception) {
			serverNotReady(visu_visuid);
		}
	}

	this.socketSend=function(data) {
		if (socket.readyState==1) {	
			var tmp=JSON.stringify(data);
			socket.send(tmp);
		}
	}


	this.request_login=function() {
		//Login per SID und visuId
		that.socketSend(['LOGIN',sid,visu_visuid]);
	}

	this.request_pong=function() {
		//Antwort auf PING
		that.socketSend(['PONG',sid]);
	}

	this.request_close=function() {
		//Verbindung schließen
		that.socketSend(['CLOSE',sid,visu_visuid]);
	}

	this.request_initPage=function(pageid,userClick) {
		//Seite/Popup aufrufen (KOs updaten)
		that.socketSend(['INITPAGE',sid,pageid,((userClick===true)?1:0)]);
	}

	this.request_closePopup=function(pageid,userClick) {
		//Popup schließen (KOs updaten)
		that.socketSend(['CLOSEPOPUP',sid,pageid,((userClick===true)?1:0)]);
	}

	this.request_execCmdList=function(elementId) {
		//CMD-Liste ausführen
		that.socketSend(['EXECCMDLIST',sid,elementId]);
	}

	this.request_setKoValue=function(koId,value) {
		//KO-Value setzen
		that.socketSend(['SETKOVALUE',sid,koId,value]);
	}
	
	this.response_initPage=function(page,currentPageIDs,background,item) {
		var renderPage=false;

		if (page.pagetyp==1) {	//Popup
			if (!focusPopup(page.id)) {
				cancelAll();
				renderPage=true;

				if (page.xpos!="" && page.ypos!="") {
					var parentWin=openWindow(page.xsize,page.ysize,page.xpos,page.ypos);
				} else {
					var parentWin=openWindow(page.xsize,page.ysize);
				}

				if (page.autoclose!=0) {
					document.getElementById(parentWin).dataset.autoclosetimer=set_timeout("closePopup('"+parentWin+"');",page.autoclose*1000,parentWin,"document.getElementById('"+parentWin+"').dataset.autoclosetimer");
				} else {
					document.getElementById(parentWin).dataset.autoclosetimer="";
				}
			}
		} else {	//normale Seite/globale Inkludeseite
			cancelAll();
			renderPage=true;

			closeAllPopups();
			clearObject("w0-page",0);
			var parentWin="w0";
		}

		if (renderPage) {
			//Page-Daten
			var obj=document.getElementById(parentWin);
			obj.dataset.pageid=page.id;
			obj.dataset.pagetyp=page.pagetyp;

			//Container der Elemente (w#-page): winId (w#) merken (wird für PopupClose gebraucht...)
			document.getElementById(parentWin+"-page").dataset.winid=parentWin;

			//Offset der Seite (Popup) bezogen auf linke/obere Ecke des Browsers
			var pageXpos=document.getElementById(parentWin+"-page").offsetLeft;
			var pageYpos=document.getElementById(parentWin+"-page").offsetTop;
	
			//Seitenhintergrund
			if (background!="") {
				document.getElementById(parentWin+"-page").style.background=background;
				document.getElementById(parentWin+"-page").style.backgroundSize="100% 100%";
				document.getElementById(parentWin+"-page").style.backgroundRepeat="no-repeat";
			} else {
				document.getElementById(parentWin+"-page").style.background="";
			}

			//Visuelemente aufbauen
			for (var t=0;t<item.length;t++) {	
				//DIV erzeugen und allgemeine Eigenschaften
				var obj=createNewDiv(parentWin+"-page","e-"+item[t].id);
				obj.dataset.id=item[t].id;
				obj.dataset.controltyp=item[t].controltyp;

				if (item[t].controltyp==24) {
					//Codeschloss: Länge des Codes
					obj.dataset.var1=item[t].var1.length;
				} else {
					obj.dataset.var1=item[t].var1;
				}
				obj.dataset.var2=item[t].var2;
				obj.dataset.var3=item[t].var3;
				obj.dataset.var4=item[t].var4;
				obj.dataset.var5=item[t].var5;
				obj.dataset.var6=item[t].var6;
				obj.dataset.var7=item[t].var7;
				obj.dataset.var8=item[t].var8;
				obj.dataset.var9=item[t].var9;
				obj.dataset.var10=item[t].var10;
				obj.dataset.gotopageid=item[t].gotopageid;
				obj.dataset.closepopup=item[t].closepopup;
				obj.dataset.hascmd=item[t].hascmd;
				obj.dataset.pagex=pageXpos;
				obj.dataset.pagey=pageYpos;
				obj.dataset.gaid=0;
				obj.dataset.gaid2=0;

				//Design merken
				obj.dataset.d_css=item[t].design.css;
				obj.dataset.d_csslast="";
				obj.dataset.d_rcss=item[t].design.rcss;
				obj.dataset.d_text=item[t].design.text;
				obj.dataset.d_ranim=item[t].design.ranim;
				obj.dataset.d_img=item[t].design.img;
				obj.dataset.d_caption="";

				//Livevalue-Update			
				obj.dataset.live=0;
				obj.dataset.livevalue="";
				obj.dataset.timeout=item[t].galive;
				obj.dataset.timeouttimer="";
				if (item[t].galive!=0 && item[t].gaid>0) {
					obj.dataset.livekoid=item[t].gaid;
				} else {
					obj.dataset.livekoid=0;
				}

				//Anklickbar?
				if (item[t].gotopageid>0 || item[t].closepopup>0 || item[t].hascmd>0) {
					obj.dataset.var0="1";
				} else {
					obj.dataset.var0="0";
				}
				
				//GAs
				if (item[t].gaid>0) {
					obj.dataset.gaid=item[t].gaid;
					obj.dataset.value=item[t].kovalue;
				}
				if (item[t].gaid2>0) {
					obj.dataset.gaid2=item[t].gaid2;
				}

				controlConstruct(obj,item[t]);
				refreshItem(item[t].design,true);
			}

		}
		cancelTimeouts();
	}

	this.response_refreshItems=function(designArr) {
		for (var t=0;t<designArr.length;t++) {
			refreshItem(designArr[t],false);
		}
	}	
}



/*
============================================================================
Visuelemente & Formulare
============================================================================
*/

function refreshItem(design,firstInit) {
	var obj=document.getElementById("e-"+design.id);
	if (obj) {

		obj.dataset.value=design.kovalue;
		obj.dataset.d_text=design.text;			
		obj.dataset.d_ranim=design.ranim;
		obj.dataset.d_img=design.img;

		if (obj.dataset.live==1) {

			if (design.css!="") {obj.dataset.d_css=design.css;}

		} else {

			obj.dataset.live=0;
			obj.dataset.livevalue="";
			
			if (design.rcss==1) {
				//Animation mit diesem Trick neustarten (falls es sich um die gleiche Animation wie zuvor handelt, wird die Animation ohne diesen Trick nicht neugestartet!)
				if (design.ranim==1) {
					element_setCss(obj,"-webkit-animation:none;");
					obj.offsetWidth;
				}

				//CSS ggf. aus dem Puffer holen bzw. merken (Server überträgt css nur bei Änderung)
				if (design.css!="") {
					element_setCss(obj,pS(design.css,design.kovalue));				
					obj.dataset.d_css=design.css;
				} else {
					element_setCss(obj,pS(obj.dataset.d_css,design.kovalue));
				}
				
				//Klicks auf unsichtbare Elemente ignorieren
				if (parseFloat(obj.style.opacity)==0) {
					obj.style.pointerEvents="none";
				}
			}
	
			//Caption ermitteln
			if (obj.dataset.controltyp==11 || obj.dataset.controltyp==13 || obj.dataset.controltyp==33) {
				var tmp=design.text.match(/\{(.*?)\}/);
				if (tmp) {
					obj.dataset.d_caption=tmp[0];
				} else {
					obj.dataset.d_caption=design.text;
				}
			}
	
			controlRefresh(obj,firstInit);
			
		}
	}
}

function execCmdList(elementId) {
	//Befehlsliste ausführen
	if (visu_socket) {visu_socket.request_execCmdList(elementId);}
}

function getKoValue(obj) {
	return ((obj.dataset.live==1)?obj.dataset.livevalue:obj.dataset.value);
}

function setKoValue(koId,value) {
	//setzt ein KO auf den Wert value
	if (koId>0) {
		if (visu_socket) {visu_socket.request_setKoValue(koId,value);}
	}
}

function setKoValue_Interval(elementId,drag,interval,unblock) {
	//setzt ein KO während der Eingabe (z.B. Drehregler) auf den Wert value, sofern eine Änderung vorliegt
	//Hinweis: der KO-Wert wird in dataset.koeditvalue erwartet (und es werden diverse dataset-Variablen angelegt)
	//drag: true=Dragging aktiv, false=Dragging beendet
	//interval: -1=deaktiviert, 0=unmittelbar setzen, 1..oo=nur alle x Millisekunden setzen
	//unblock (optional): wird nur intern verwendet
	var d=document.getElementById("e-"+elementId);
	if (d) {
		setLiveValue(elementId,drag);
		if (!drag || unblock===true) {element_drag.kotimer=clear_timeout(element_drag.kotimer);}
		if (!drag || (interval>=0 && (unblock===true || element_drag.koblock==0))) {
			element_drag.koblock=0;
			if (element_drag.kolastvalue+"x"!==element_drag.koeditvalue+"x") {	//String-Vergleich...
				setKoValue(d.dataset.gaid2,element_drag.koeditvalue);
				element_drag.kolastvalue=element_drag.koeditvalue;
			}
			if (drag && interval>0) {
				element_drag.koblock=1;
				element_drag.kotimer=set_timeout("setKoValue_Interval("+elementId+",true,"+interval+",true);",interval,d.id,"element_drag.kotimer");
			}			
		}
	}
}

function setLiveValue(elementId,drag,unblock) {
	var interval=50;	//Throttle-Intervall
	var senderObj=document.getElementById("e-"+elementId);
	if (senderObj) {
		if (!drag || unblock===true) {element_drag.livetimer=clear_timeout(element_drag.livetimer);}

		if (!drag || (interval>=0 && (unblock===true || element_drag.liveblock==0))) {
			element_drag.liveblock=0;

			var koId=parseInt(senderObj.dataset.gaid);
			if (koId>0) {
				var obj=document.getElementById("windowContainer").querySelectorAll("[data-livekoid='"+koId+"']");
				if (drag) {
					for (var t=0; t<obj.length; t++) {
						obj[t].dataset.live=1;
						obj[t].dataset.livevalue=element_drag.koeditvalue;
						obj[t].dataset.timeouttimer=clear_timeout(obj[t].dataset.timeouttimer);
						element_setCss(obj[t],pS(obj[t].dataset.d_css,element_drag.koeditvalue));
						controlRefresh(obj[t],false);	
					}
				} else {
					for (var t=0; t<obj.length; t++) {
						if (obj[t].dataset.timeout>0) {
							obj[t].dataset.live=1;
							obj[t].dataset.livevalue=element_drag.koeditvalue;
							element_setCss(obj[t],pS(obj[t].dataset.d_css,element_drag.koeditvalue));
							controlRefresh(obj[t],false);
							obj[t].dataset.timeouttimer=clear_timeout(obj[t].dataset.timeouttimer);
							obj[t].dataset.timeouttimer=set_timeout("setLiveValue_timeout("+obj[t].dataset.id+")",obj[t].dataset.timeout,obj[t].id,"document.getElementById('"+obj[t].id+"').dataset.timeouttimer");
						} else {
							obj[t].dataset.live=0;
							obj[t].dataset.livevalue="";
							var design={id:obj[t].dataset.id,kovalue:obj[t].dataset.value,rcss:obj[t].dataset.d_rcss,css:obj[t].dataset.d_css,caption:obj[t].dataset.d_caption,text:obj[t].dataset.d_text,ranim:obj[t].dataset.d_ranim,img:obj[t].dataset.d_img};	
							refreshItem(design,false);
						}
					}
				}
			}	

			if (!drag) {
				element_drag.kotimer=clear_timeout(element_drag.kotimer);
				element_drag.liveblock=0;
				element_drag.livetimer=clear_timeout(element_drag.livetimer);
			} else if (interval>0) {
				element_drag.liveblock=1;
				element_drag.livetimer=clear_timeout(element_drag.livetimer);
				element_drag.livetimer=set_timeout("setLiveValue("+elementId+",true,true);",interval,senderObj.id,"element_drag.livetimer");
			}			
		}
	}
}

function setLiveValue_timeout(elementId) {
	var obj=document.getElementById("e-"+elementId);
	if (obj) {
		obj.dataset.live=0;
		obj.dataset.livevalue="";
		var design={id:obj.dataset.id,kovalue:obj.dataset.value,rcss:obj.dataset.d_rcss,css:obj.dataset.d_css,caption:obj.dataset.d_caption,text:obj.dataset.d_text,ranim:obj.dataset.d_ranim,img:obj.dataset.d_img};
		refreshItem(design,false);
	}
}

function resetKoValue_Interval(lastValue) {
	element_drag.kotimer=clear_timeout(element_drag.kotimer);
	element_drag.koeditvalue="";
	element_drag.kolastvalue=lastValue;
	element_drag.koblock=0;
}

function element_setCss(obj,css) {
	if (visu_cssbuffer) {
		if (obj.dataset.d_csslast!=css) {
			obj.dataset.d_csslast=css;
			obj.style.cssText=css;
		}
	} else {
		obj.style.cssText=css;
	}
}

function controlGetFormData(formId) {
	//bildet aus allen Controls in dem "Formular" formId einen String ("1;2;3;...;")
	//die Controls müssen alle eine ID in der Art "x-y-fd#" haben, wobei # eine laufende Nummer ist (fd1,fd2,...)
	//die Reihenfolge ist egal, aus fd1=1,fd3=3,fd2=2 wird "1;2;3;" (NICHT "1;3;2;")
	//fehlende Nummern werden im String aufgefüllt: aus fd1=1,fd2=2,fd4=4 wird "1;2;;4;"
	var n=new Array();
	var tmp=document.getElementById(formId).querySelectorAll("[data-type]");
	for (var t=0; t<tmp.length; t++) {
		var type=tmp[t].dataset.type;
		if (tmp[t].id) {var id=tmp[t].id.split("-")[2].replace("fd","");}
		if (type==1)  {n[id]=stringCleanup(tmp[t].value);}				//Input (Text/Hidden)
		if (type==5)  {n[id]=stringCleanup(tmp[t].dataset.value);}		//Checkbox
		if (type==6)  {n[id]=stringCleanup(tmp[t].dataset.value);}		//Selectbox zum Durchklicken (für TSU)
	}
	var r="";
	for (var t=0; t<n.length; t++) {
		if (n[t]==undefined) {n[t]="";}
		r+=n[t]+AJAX_SEPARATOR1;
	}
	return r;
}

function checkboxInit(obj) {
	if (obj.dataset.value!=0) {
		obj.innerHTML="<button class='elementContent' style='width:100%; height:100%; background:"+visu_indiColor+"; color:"+visu_indiColorText+";'>"+obj.dataset.list.split("|")[1]+"</button>";
	} else {
		obj.innerHTML=obj.dataset.list.split("|")[0];
	}
}

function checkboxToggle(obj) {
	if (obj.dataset.value!=0) {
		obj.dataset.value=0;
	} else {
		obj.dataset.value=1;
	}
	checkboxInit(obj);
}

function selectboxInit(obj) {
	if (obj) {
		var n="";
		var id=obj.dataset.value;
		var list=obj.dataset.list.split(";");
	
		n+="<select onChange='document.getElementById(\""+obj.id+"\").dataset.value=this.value;' class='controlSelect'>";
		for (var t=0;t<(list.length-1);t++) {
			
			var tmp=list[t].split("|")[1];
			tmp=tmp.replace(/</g,"&lt;");
			tmp=tmp.replace(/>/g,"&gt;");
	
			if (id==list[t].split("|")[0]) {
				n+="<option value='"+list[t].split("|")[0]+"' selected>"+tmp+"</option>";
			} else {
				n+="<option value='"+list[t].split("|")[0]+"'>"+tmp+"</option>";
			}
		}
		n+="</select>";
		obj.innerHTML=n;
	}
}

function scrollToTop(objId) {
	var obj=document.getElementById(objId);
	if (obj) {obj.scrollTop=0;}
}

function scrollToEnd(objId) {
	var obj=document.getElementById(objId);
	if (obj) {obj.scrollTop=obj.scrollHeight;}
}

function scrollUp(objId) {
	var obj=document.getElementById(objId);
	if (obj) {obj.scrollTop-=parseInt(obj.clientHeight);}
}

function scrollDown(objId) {
	var obj=document.getElementById(objId);
	if (obj) {obj.scrollTop+=parseInt(obj.clientHeight);}
}

function showShortClickItem(obj) {
	//blendet einen Klick-Indikator für 200 Millisekunden ein
	if (obj) {
		if (obj.dataset.shortclicktimer===undefined) {obj.dataset.shortclicktimer="";}
		obj.dataset.shortclicktimer=clear_timeout(obj.dataset.shortclicktimer);
		obj.dataset.shortclicktimer=set_timeout("showShortClickItemReset('"+obj.id+"');",200,obj.id,"document.getElementById('"+obj.id+"').dataset.shortclicktimer");
		obj.className="shortClick";
	}
}

function showShortClickItemReset(elementId) {
	var obj=document.getElementById(elementId);
	if (obj) {
		obj.className="";
	}
}

function startLongClick(obj,func,visible) {
	if (obj) {
		if (visible===true) {
			var d=document.getElementById("longclick");
			d.style.display="inline";
			d.style.borderColor=visu_indiColor;
			d.style.borderTopColor=visu_indiColorText;
			
			var tmp=obj.getBoundingClientRect();
			if (tmp.width>tmp.height) {var w=tmp.height;} else {var w=tmp.width;}
			d.style.width=(w-6)+'px';
			d.style.height=(w-6)+'px';
			var x=parseFloat((tmp.left+window.scrollX)+tmp.width/2)-(w/2);
			var y=parseFloat((tmp.top+window.scrollY)+tmp.height/2)-(w/2);
			d.style.left=x+'px';
			d.style.top=y+'px';
		}
		longClickTimer=set_timeout(func,1000,false,"longClickTimer");
	}
}

function cancelLongClick() {
	longClickTimer=clear_timeout(longClickTimer);
	var d=document.getElementById("longclick");
	if (d) {d.style.display="none";}
}

function visuElementClick(elementId,hideClick) {
	var obj=document.getElementById("e-"+elementId);
	if (!hideClick) {showShortClickItem(obj);}

	if (obj.dataset.hascmd>0) { 			//Befehle
		execCmdList(obj.dataset.id);
	}
	if (obj.dataset.gotopageid>0) { 		//Seitenwechsel
		openPage(obj.dataset.gotopageid,false,true);
	}
	if (obj.dataset.closepopup>0) { 		//Popup schliessen
		closePopup(obj.parentNode.dataset.winid,true);
	}

	clickCancel();
}

function getMouseValue(obj,canvas,isStart,koValue,format,coord,modus,minX,maxX,minY,maxY,minA,maxA,vlist,vstep,vsnap,flip) {
	//obj: Visuelement
	//canvas: Maus-aktiver Bereich des Visuelements (muss nicht wirklich ein "canvas"-Element sein)
	//isStart: true=erster Klick (KO parsen und Anfasserposition merken)
	//koValue: (nur bei isStart=true notwendig) Default-KO-Wert
	//format: 0=X+Y (KO-Format: "X,Y"), 1=nur X, 2=nur Y
	//coord: 0=kartesisch, 1=polar
	//modus: 0=absolut, 1=relativ, 2=Inkremental (nur X und coord muss 1 sein)
	//koValue: KO-Wert, ungeparsed (nur bei modus!=0 sinnvoll)
	//minXY/maxXY: leer=Default (VE-Maße bzw. 0..360), -oo..oo=Range
	//minA/maxA: Winkelbereich bei polar (jeweils 0..360, min<max)
	//vlist: Anzahl der Nachkommastellen
	//vstep: Raster
	//vsnap: (nur bei Inkrementalgeber) Anzahl der Rastungen (360 muss restlos durch vsnap teilbar sein)
	//flip: kehrt die Mauskoordinaten um
	
	if (!event) {var event=window.event;}
	
	if (obj && canvas) {

		pos=element_getMousePosition(obj,canvas,flip);
		if (pos===false) {return false;}

		//Min/Max-Defaultwerte ermitteln
		vstep=parseFloat(vstep);
		vlist=parseInt(vlist);
		vsnap=parseInt(vsnap);
		if (pos.w>=pos.h) {var maxR=pos.h/2;} else {var maxR=pos.w/2;}
		if (isNaN(parseFloat(minA)) || modus==2) {minA=0;}
		if (isNaN(parseFloat(maxA)) || modus==2) {maxA=360;}
		if (isNaN(parseFloat(minX))) {
			if (coord==0) {minX=0;} else {minX=minA;}
		}
		if (isNaN(parseFloat(maxX))) {
			if (coord==0) {maxX=pos.w;} else {maxX=maxA;}
		}
		if (isNaN(parseFloat(minY))) {
			if (coord==0) {minY=0;} else {minY=0;}
		}
		if (isNaN(parseFloat(maxY))) {
			if (coord==0) {maxY=pos.h;} else {maxY=maxR;}
		}
		minX=parseFloat(minX);
		maxX=parseFloat(maxX);
		minY=parseFloat(minY);
		maxY=parseFloat(maxY);
		minA=parseFloat(minA);
		maxA=parseFloat(maxA);

		if (isStart) {
			//KO-Wert beim Start parsen und merken
			var tmp=koValue.toString().split(",");
			if (isNaN(parseFloat(tmp[0]))) {
				tmp[0]=minX;
			}
			if (format==2) {
				if (tmp.length<2) {tmp[1]=tmp[0];}
			} else if (tmp.length<2 || isNaN(parseFloat(tmp[1]))) {
				tmp[1]=minY;
			}
			element_drag.startx=pos.x;
			element_drag.starty=pos.y;
			element_drag.valuex=parseFloat(tmp[0]);
			element_drag.valuey=parseFloat(tmp[1]);
			element_drag.lastcx="";
			return true;
		}
			
		//Cursor und Value ermitteln
		if (coord==0) {
			if (modus==0) {
				var x=(pos.x/pos.w)*Math.abs(maxX-minX)+minX;
				var cursorX=pos.x;
				var y=(pos.y/pos.h)*Math.abs(maxY-minY)+minY;
				var cursorY=pos.y;

			} else if (modus==1) {
				defaultvalue=valueRange(element_drag.valuex,element_drag.valuey);
				var x=((pos.x-element_drag.startx)/pos.w)*Math.abs(maxX-minX)+defaultvalue.x;
				var cursorX=pos.w*((x-minX)/Math.abs(maxX-minX));
				var y=((pos.y-element_drag.starty)/pos.h)*Math.abs(maxY-minY)+defaultvalue.y;
				var cursorY=pos.h*((y-minY)/Math.abs(maxY-minY));
			}
			
		} else {
			if (modus==0) {
				var tmp=parseFloat((Math.atan2(-(pos.x-pos.w/2),-(pos.y-pos.h/2))-Math.PI)*(-180/Math.PI));
				var x=((tmp-minA)/Math.abs(maxA-minA))*Math.abs(maxX-minX)+minX;
				var cursorX=tmp;				
				var tmp=parseFloat(Math.sqrt(Math.pow(pos.x-pos.w/2,2)+Math.pow(pos.y-pos.h/2,2)));
				var y=(tmp/maxR)*Math.abs(maxY-minY)+minY;
				var cursorY=maxR*((y-minY)/Math.abs(maxY-minY));

			} else if (modus==1) {
				defaultvalue=valueRange(element_drag.valuex,element_drag.valuey);
				var tmp1=parseFloat((Math.atan2(-(element_drag.startx-(pos.w/2)),-(element_drag.starty-(pos.h/2)))-Math.PI)*(-180/Math.PI));
				var tmp2=((defaultvalue.x-minX)/Math.abs(maxX-minX))*Math.abs(maxA-minA)+minA;
				var tmp=parseFloat((Math.atan2(-(pos.x-pos.w/2),-(pos.y-pos.h/2))-Math.PI+((tmp1-tmp2)/180*Math.PI))*(-180/Math.PI));
				if (tmp<0 && tmp>-0.00001) {tmp=0;}	//Rundungsfehler (Float) ausgleichen
				if (tmp<0) {tmp=360+tmp;}
				if (tmp>360) {tmp=tmp-360;}
				var x=((tmp-minA)/Math.abs(maxA-minA))*Math.abs(maxX-minX)+minX;
				var cursorX=tmp;				
				var tmp=Math.sqrt(Math.pow(pos.x-pos.w/2,2)+Math.pow(pos.y-pos.h/2,2))-Math.sqrt(Math.pow(element_drag.startx-(pos.w/2),2)+Math.pow(element_drag.starty-(pos.h/2),2));
				var y=(tmp/maxR)*Math.abs(maxY-minY)+defaultvalue.y;
				var cursorY=maxR*((y-minY)/Math.abs(maxY-minY));

			} else if (modus==2) {
				var tmp1=parseFloat((Math.atan2(-(element_drag.startx-(pos.w/2)),-(element_drag.starty-(pos.h/2)))-Math.PI)*(-180/Math.PI));
				var tmp2=parseFloat((Math.atan2(-(pos.x-pos.w/2),-(pos.y-pos.h/2))-Math.PI)*(-180/Math.PI));
				tmp1=parseInt(tmp1/vsnap)*vsnap;
				tmp2=parseInt(tmp2/vsnap)*vsnap;
				var tmp=tmp2-tmp1;
				var tmpStep=0;
				
				if ((tmp>0 && tmp<180) || tmp<-180) {
					tmpStep=1;
				} else if ((tmp<0 && tmp>-180) || tmp>180) {
					tmpStep=-1;
				}

				if (!isNaN(vstep) && vstep!=0) {
					tmpStep*=vstep;
				} else if (vlist>=0) {
					tmpStep/=(Math.pow(10,vlist));
				}

				var x=element_drag.valuex;
				if (tmpStep!=0) {
					x+=tmpStep;
					var tmp=valueRange(x,0);
					element_drag.valuex=tmp.x;
					element_drag.startx=pos.x;
					element_drag.starty=pos.y;
				}

				var tmp=parseFloat((Math.atan2(-(pos.x-pos.w/2),-(pos.y-pos.h/2))-Math.PI)*(-180/Math.PI));
				var cursorX=parseInt(tmp/vsnap)*vsnap;

				var y=0;
				var cursorY=0;
			}
		}
						
		if (coord==0) {
			if (cursorX<0) {cursorX=0;}
			if (cursorX>pos.w) {cursorX=pos.w;}
			if (cursorY<0) {cursorY=0;}
			if (cursorY>pos.h) {cursorY=pos.h;}
		} else {
			if (cursorX<minA) {cursorX=minA;}
			if (cursorX>maxA) {cursorX=maxA;}
			if (cursorY<0) {cursorY=0;}
			if (cursorY>maxR) {cursorY=maxR;}
		}
		
		//Polar: Anschlag (nur Winkel)
		if (coord==1 && modus!=2) {
			if (element_drag.lastcx=="") {
				element_drag.lastcx=cursorX;
			} else {
				var tmp=parseFloat(element_drag.lastcx)-cursorX;
				if (tmp>=Math.abs(maxA-minA)/2) {
					x=maxX;
					cursorX=maxA;
				} else if (tmp<=-Math.abs(maxA-minA)/2) {
					x=minX;
					cursorX=minA;
				} else {
					element_drag.lastcx=cursorX;
				}
			}
		}		

		value=valueRange(x,y);

		//Raster
		if (!isNaN(vstep) && vstep!=0) {
			value.x=Math.round(value.x/vstep)*vstep;
			value.y=Math.round(value.y/vstep)*vstep;
		}

		//Nachkommastellen
		if (vlist>=0) {
			value.x=value.x.toFixed(vlist);
			value.y=value.y.toFixed(vlist);
		}

		return {valuex:value.x,valuey:value.y,cursorx:cursorX,cursory:cursorY,objw:pos.w,objh:pos.h};
	}
	return false;
	
	function valueRange(x,y) {
		x=parseFloat(x);
		y=parseFloat(y);
		if (x<minX) {x=minX;}
		if (x>maxX) {x=maxX;}
		if (y<minY) {y=minY;}
		if (y>maxY) {y=maxY;}
		return {x:x,y:y};
	}
}
